# CSS Foo Module Level 1

## Editor’s Draft, 1 January 1970

Specification Metadata

This version:
  https://drafts.csswg.org/css-foo/
Latest published version:
  https://www.w3.org/TR/css-foo/
Editors:
  Name1 (Company1)
  Name2 (Company2)
Issue Tracking:
  GitHub Issues
Suggest an Edit for this Spec:
  GitHub Editor

Copyright © 1970 W3C® (MIT, ERCIM, Keio, Beihang). W3C liability, trademark and permissive document license rules apply.

## Abstract

This module introduces the foo property and related values, which do bar and baz.

CSS is a language for describing the rendering of structured documents (such as HTML and XML) on screen, on paper, etc.

## Status of this document

This is a public copy of the editors’ draft. It is provided for discussion only and may change at any moment. Its publication here does not imply endorsement of its contents by W3C. Don’t cite this document other than as work in progress.

GitHub Issues are preferred for discussion of this specification. When filing an issue, please put the text “css-foo” in the title, preferably like this: “[css-foo] …summary of comment…”. All issues and comments are archived, and there is also a historical archive.

This document was produced by the CSS Working Group.

This document was produced by a group operating under the W3C Patent Policy. W3C maintains a public list of any patent disclosures made in connection with the deliverables of the group; that page also includes instructions for disclosing a patent. An individual who has actual knowledge of a patent which the individual believes contains Essential Claim(s) must disclose the information in accordance with section 6 of the W3C Patent Policy.

This document is governed by the 1 March 2019 W3C Process Document.

## 1. Introduction

This section is not normative.

Due to the need to have example specifications, the CSS community found a great need to have a foo property. This specifications addresses this need in a very simply way. While it provides a very limited set of features for authors, it effectively demonstrates how to write a CSS specification.

## 2. Sample section

Look at the mark-up in this section for examples. See the documentation for more precise instructions. The companion of this template shows the processed result.

To refer to HTML, use [HTML] (note the double square brackets in the source). To make a normative reference, insert a “!”, like this: [CSS-SYNTAX-3] The currently available list of bibliographic data can of course be extended.

We write about a property such as foo like this and about a piece of CSS code like this: foo: bar. (Note that if it looks like a "property: value", it’ll automatically link to the property.) Inline HTML and XML are similar, but use the CODE element: `<blockquote>...</blockquote>` and `<r:xyz>...</r:xyz>`.

Note: Note that the property will automatically be linked to its definition.

To define terms into the index, there are many variants, but hopefully the simplest will be the most common. Note that you need to explicitly export any plain `dfn`s you want to be linkable from other specs, but all other types of definition automatically export themselves.

Note: Note that you can add non-normative notes like this.

Of course, multi-paragraph notes are also possible: just enclose them in a `div`:

Note that this note is a multi-paragraph note.

It doesn’t need to have two paragraphs, but it could.

A longer note

When you want to insert a longer note to provide some useful explanation, but the note itself is not critical to the section it’s placed in, use a `details` note instead.

This will hide the note by default, so it’s less distracting to the flow of the section. (At least, in browsers that support `details`; legacy browsers will get something like a normal note.)

Displayed examples come in eight different types: CSS examples that need no separate paragraph of explanation are put in a simple PRE:

    EM { font-style: italic }

CSS examples that need extra text need a DIV.

The following example is the same as the previous one, but now it is explained:

    EM { font-style: italic }

Illegal CSS examples (examples of errors) are the same, but with class "illegal example". Examples of HTML and XML code have class "html" and "xml" respectively, but are otherwise ditto.

A formal definition of a property looks like this:

### 2.1. Internal display model: the foo property

Name: | foo
Value: | inline-inside | block-inside | table | ruby | icon
Initial: | text
Applies to: | all elements
Inherited: | no
Percentages: | n/a
Computed value: | specified value
Canonical order: | per grammar
Animatable: | no

value-name
  Define values using a `dl`. Note that the `dl` wrapper specifies the default `dfn` type and what property it’s for, so you don’t have to repeat that information in each individual definition.

Cross-references are created by enclosing a term or phrase in <dfn> (like the word cross-references earlier in this sentence). Then an <a> without an `href=""` attribute with the same text content will automatically be linked. Both <dfn>s and <a>s are typed, which allows the same text to appear as different types of terms without a collision. The type can often be inferred, but sometimes it needs to be specified, like when you’re linking to a `Foo` WebIDL interface. (Here, we’re using the IDL linking shorthand to make it clear that this is one of the IDL types.)

And a figure with a caption is done like this:

Just a random image. Use SVG if you can. Otherwise, W3C prefers PNG over GIF (obviously, since PNG is a W3C Rec).

Don’t forget to write the alt.

An open issue or editorial remark is OK in a WD, but they should be resolved/removed before the document goes to “CR” (Candidate Recommendation). Use `class="issue"` on an element, or begin a paragraph with “Issue:”.

Inline issues will be copied into an Issues Index at the end of the document, for easy reference.

    /* Write WebIDL in a <pre class="idl"> as plain text. */
    interface Foo {
      readonly attribute CSSOMString bar;
      boolean baz(FooDict Arg1, (CSSOMString or Foo) Arg2);
    };

    dictionary FooDict {
      sequence<Foo> foos;
      boolean bar;
      CSSOMString baz = "qux";
    };

### 2.2. Shorthands and Descriptors

Shorthand properties have a smaller set of values to provide:

Name: | shorthand-foo
Value: | foo | bar | baz
Initial: | see individual properties
Applies to: | see individual properties
Inherited: | see individual properties
Percentages: | see individual properties
Computed value: | see individual properties
Animation type: | see individual properties
Canonical order: | per grammar

Adding new values to an existing property? Use a partial:

Name: | foo
New values: | another-icon

Or when defining a descriptor, use a descdef block (partials work here, too):

Name: | descriptor-foo
For: | @some-at-rule
Value: | more | values
Initial: | values

(A required descriptor can use `Initial: n/a.)

## Privacy and Security Considerations

Make some considerations about privacy and security.

## Conformance

### Document conventions

Conformance requirements are expressed with a combination of descriptive assertions and RFC 2119 terminology. The key words “MUST”, “MUST NOT”, “REQUIRED”, “SHALL”, “SHALL NOT”, “SHOULD”, “SHOULD NOT”, “RECOMMENDED”, “MAY”, and “OPTIONAL” in the normative parts of this document are to be interpreted as described in RFC 2119. However, for readability, these words do not appear in all uppercase letters in this specification.

All of the text of this specification is normative except sections explicitly marked as non-normative, examples, and notes. [RFC2119]

Examples in this specification are introduced with the words “for example” or are set apart from the normative text with `class="example"`, like this:

This is an example of an informative example.

Informative notes begin with the word “Note” and are set apart from the normative text with `class="note"`, like this:

Note, this is an informative note.

Advisements are normative sections styled to evoke special attention and are set apart from other normative text with `<strong class="advisement">`, like this: UAs MUST provide an accessible alternative.

### Conformance classes

Conformance to this specification is defined for three conformance classes:

style sheet
  A CSS style sheet.
renderer
  A UA that interprets the semantics of a style sheet and renders documents that use them.
authoring tool
  A UA that writes a style sheet.

A style sheet is conformant to this specification if all of its statements that use syntax defined in this module are valid according to the generic CSS grammar and the individual grammars of each feature defined in this module.

A renderer is conformant to this specification if, in addition to interpreting the style sheet as defined by the appropriate specifications, it supports all the features defined by this specification by parsing them correctly and rendering the document accordingly. However, the inability of a UA to correctly render a document due to limitations of the device does not make the UA non-conformant. (For example, a UA is not required to render color on a monochrome monitor.)

An authoring tool is conformant to this specification if it writes style sheets that are syntactically correct according to the generic CSS grammar and the individual grammars of each feature in this module, and meet all other conformance requirements of style sheets as described in this module.

### Requirements for Responsible Implementation of CSS

The following sections define several conformance requirements for implementing CSS responsibly, in a way that promotes interoperability in the present and future.

#### Partial Implementations

So that authors can exploit the forward-compatible parsing rules to assign fallback values, CSS renderers must treat as invalid (and ignore as appropriate) any at-rules, properties, property values, keywords, and other syntactic constructs for which they have no usable level of support. In particular, user agents must not selectively ignore unsupported property values and honor supported values in a single multi-value property declaration: if any value is considered invalid (as unsupported values must be), CSS requires that the entire declaration be ignored.

#### Implementations of Unstable and Proprietary Features

To avoid clashes with future stable CSS features, the CSSWG recommends following best practices for the implementation of unstable features and proprietary extensions to CSS.

#### Implementations of CR-level Features

Once a specification reaches the Candidate Recommendation stage, implementers should release an unprefixed implementation of any CR-level feature they can demonstrate to be correctly implemented according to spec, and should avoid exposing a prefixed variant of that feature.

To establish and maintain the interoperability of CSS across implementations, the CSS Working Group requests that non-experimental CSS renderers submit an implementation report (and, if necessary, the testcases used for that implementation report) to the W3C before releasing an unprefixed implementation of any CSS features. Testcases submitted to W3C are subject to review and correction by the CSS Working Group.

Further information on submitting testcases and implementation reports can be found from on the CSS Working Group’s website at http://www.w3.org/Style/CSS/Test/. Questions should be directed to the public-css-testsuite@w3.org mailing list.

## Index

### Terms defined by this specification

- bar
  - attribute for Foo, in §2.1
  - dict-member for FooDict, in §2.1
- baz, in §2.1
- baz(Arg1, Arg2), in §2.1
- Cross-references, in §2.1
- descriptor-foo, in §2.2
- Foo, in §2.1
- foo, in §2.1
- FooDict, in §2.1
- foos, in §2.1
- index, in §2
- shorthand-foo, in §2.2
- simplest, in §2
- terms, in §2
- value-name, in §2.1
- variant, in §2

https://drafts.csswg.org/css-values-4/#comb-one Referenced in:

- 2.1. Internal display model: the foo property (2) (3) (4)
- 2.2. Shorthands and Descriptors (2) (3)

https://drafts.csswg.org/cssom-1/#cssomstring Referenced in:

- 2.1. Internal display model: the foo property (2) (3)

https://html.spec.whatwg.org/multipage/interactive-elements.html#the-details-element Referenced in:

- 2. Sample section (2)

https://html.spec.whatwg.org/multipage/text-level-semantics.html#the-dfn-element Referenced in:

- 2. Sample section
- 2.1. Internal display model: the foo property

https://html.spec.whatwg.org/multipage/grouping-content.html#the-div-element Referenced in:

- 2. Sample section

https://html.spec.whatwg.org/multipage/grouping-content.html#the-dl-element Referenced in:

- 2.1. Internal display model: the foo property (2)

https://heycam.github.io/webidl/#idl-boolean Referenced in:

- 2.1. Internal display model: the foo property (2)

### Terms defined by reference

- [css-values-4] defines the following terms:
  - |
- [cssom-1] defines the following terms:
  - CSSOMString
- [HTML] defines the following terms:
  - details
  - dfn
  - div
  - dl
- [WebIDL] defines the following terms:
  - boolean

## References

### Normative References

[CSS-SYNTAX-3]
  Tab Atkins Jr.; Simon Sapin. CSS Syntax Module Level 3. 20 February 2014. CR. URL: https://www.w3.org/TR/css-syntax-3/
[CSS-VALUES-4]
  Tab Atkins Jr.; Elika Etemad. CSS Values and Units Module Level 4. 31 January 2019. WD. URL: https://www.w3.org/TR/css-values-4/
[CSSOM-1]
  Simon Pieters; Glenn Adams. CSS Object Model (CSSOM). 17 March 2016. WD. URL: https://www.w3.org/TR/cssom-1/
[HTML]
  Anne van Kesteren; et al. HTML Standard. Living Standard. URL: https://html.spec.whatwg.org/multipage/
[RFC2119]
  S. Bradner. Key words for use in RFCs to Indicate Requirement Levels. March 1997. Best Current Practice. URL: https://tools.ietf.org/html/rfc2119
[WebIDL]
  Boris Zbarsky. Web IDL. 15 December 2016. ED. URL: https://heycam.github.io/webidl/

## Property Index

Name | Value | Initial | Applies to | Inh. | %ages | Animatable | Animation type | Canonical order | Computed value
--- | --- | --- | --- | --- | --- | --- | --- | --- | ---
foo | inline-inside | block-inside | table | ruby | icon | text | all elements | no | n/a | no |  | per grammar | specified value
shorthand-foo | foo | bar | baz | see individual properties | see individual properties | see individual properties | see individual properties |  | see individual properties | per grammar | see individual properties

### @some-at-rule Descriptors

Name | Value | Initial
--- | --- | ---
descriptor-foo | more | values | values

## IDL Index

    /* Write WebIDL in a <pre class="idl"> as plain text. */
    interface Foo {
      readonly attribute CSSOMString bar;
      boolean baz(FooDict Arg1, (CSSOMString or Foo) Arg2);
    };

    dictionary FooDict {
      sequence<Foo> foos;
      boolean bar;
      CSSOMString baz = "qux";
    };

## Issues Index

An open issue or editorial remark is OK in a WD, but they should be resolved/removed before the document goes to “CR” (Candidate Recommendation). Use `class="issue"` on an element, or begin a paragraph with “Issue:”. ↵
Inline issues will be copied into an Issues Index at the end of the document, for easy reference. ↵
Make some considerations about privacy and security. ↵
#propdef-foo Referenced in:

- 1. Introduction
- 2. Sample section (2)
- 2.1. Internal display model: the foo property
- 2.2. Shorthands and Descriptors

#cross-references Referenced in:

- 2.1. Internal display model: the foo property

#foo Referenced in:

- 2.1. Internal display model: the foo property (2) (3)

#dictdef-foodict Referenced in:

- 2.1. Internal display model: the foo property